@@ -59,13 +59,13 @@ def loadStructure(filename, fmt='auto', **kw):
     rv = p.parseFile(filename)
     return rv
 
-
-# unit tests
-def test():
-    '''Execute all unit tests for the diffpy.Structure package.
-    Return a unittest TestResult object.
-    '''
-    from diffpy.Structure.tests import test
-    return test()
+# silence pyflakes checker
+assert (StructureFormatError and
+        LatticeError and SymmetryError and IsotropyError)
+assert Atom
+assert Lattice
+assert Structure
+assert PDFFitStructure
+assert __version__ or True
 
 # End of file
