@@ -104,10 +104,11 @@ class Lattice(object):
                              base vectors
         Lattice(lat)      -- create a copy of existing Lattice lat
         """
-        # check arguments
-        if baserot is not None and base is not None:
-            emsg = "Cannot use both baserot and base arguments."
-            raise ValueError(emsg)
+        # build a set of provided argument names for later use.
+        apairs = (('a', a), ('b', b), ('c', c),
+                  ('alpha', alpha), ('beta', beta), ('gamma', gamma),
+                  ('baserot', baserot), ('base', base))
+        argset = set(n for n, v in apairs if v is not None)
         # initialize data members, they values will be set by setLatPar()
         self._a = self._b = self._c = None
         self._alpha = self._beta = self._gamma = None
@@ -122,17 +123,23 @@ class Lattice(object):
         self.normbase = self.recnormbase = None
         # work out argument variants
         # Lattice()
-        if [a, b, c, alpha, beta, gamma, base] == 7*[None]:
+        if not argset:
             self.setLatPar(1.0, 1.0, 1.0, 90.0, 90.0, 90.0, baserot)
         # Lattice(base=abc)
         elif base is not None:
+            if len(argset) > 1:
+                raise ValueError("'base' must be the only argument.")
             self.setLatBase(base)
         # Lattice(lat)
         elif isinstance(a, Lattice):
-            self.setLatPar(a.a, a.b, a.c, a.alpha, a.beta, a.gamma,
-                    baserot=a.baserot)
+            if len(argset) > 1:
+                raise ValueError("Lattice object must be the only argument.")
+            self.__dict__.update(a.__dict__)
         # otherwise do default Lattice(a, b, c, alpha, beta, gamma)
         else:
+            abcabg = ('a', 'b', 'c', 'alpha', 'beta', 'gamma')
+            if not argset.issuperset(abcabg):
+                raise ValueError("Provide all 6 cell parameters.")
             self.setLatPar(a, b, c, alpha, beta, gamma, baserot=baserot)
         return
 
@@ -296,21 +303,42 @@ class Lattice(object):
     def dot(self, u, v):
         """Return dot product of 2 lattice vectors.
         """
-        dp = numpy.dot(u, numpy.dot(self.metrics, v))
+        dp = (u * numpy.dot(v, self.metrics)).sum(axis=-1)
         return dp
 
 
-    def norm(self, u):
-        """Return norm of the specified lattice vector.
+    def norm(self, xyz):
+        """Calculate norm of a lattice vector.
+
+        xyz  -- vector or an N-by-3 array of fractional coordinates.
+
+        Return float or an array of the same length as xyz.
         """
-        # CLF - duplicated code from dot for the sake of speed
-        return math.sqrt(numpy.dot(u, numpy.dot(self.metrics, u)))
+        # this is a few percent faster than sqrt(dot(u, u)).
+        return numpy.sqrt((self.cartesian(xyz)**2).sum(axis=-1))
+
+
+    def rnorm(self, hkl):
+        """Calculate norm of a reciprocal vector.
+
+        hkl  -- vector or an N-by-3 array of reciprocal coordinates.
+
+        Return float or an array of the same length as hkl.
+        """
+        hklcartn = numpy.dot(hkl, self.recbase.T)
+        return numpy.sqrt((hklcartn**2).sum(axis=-1))
 
 
     def dist(self, u, v):
-        """Return distance of 2 points in lattice coordinates.
+        """Return distance between 2 points in lattice coordinates.
+
+        u    -- vector or N-by-3 matrix of fractional coordinates.
+        v    -- vector or N-by-3 matrix of fractional coordinates.
+                Sizes of u, v must match when both of them are matrices.
+
+        Return float or an array of the same length as the matrix.
         """
-        duv = numpy.array(u) - v
+        duv = numpy.asarray(u) - v
         return self.norm(duv)
 
 
@@ -319,8 +347,14 @@ class Lattice(object):
         """
         ca = self.dot(u, v)/( self.norm(u)*self.norm(v) )
         # avoid round-off errors that would make abs(ca) greater than 1
-        ca = max(min(ca, 1), -1)
-        return math.degrees(math.acos(ca))
+        if numpy.isscalar(ca):
+            ca = max(min(ca, 1), -1)
+            rv = math.degrees(math.acos(ca))
+        else:
+            ca[ca < -1] = -1
+            ca[ca > +1] = +1
+            rv = numpy.degrees(numpy.arccos(ca))
+        return rv
 
 
     def isanisotropic(self, umx):
