@@ -18,10 +18,12 @@
 
 import unittest
 import numpy
+import numpy.linalg as numalg
 
 from diffpy.Structure import Lattice, LatticeError
 
 ##############################################################################
+
 class TestLattice(unittest.TestCase):
     """test methods of Lattice class"""
 
@@ -39,6 +41,27 @@ class TestLattice(unittest.TestCase):
         return
 
 
+    def test___init__(self):
+        '''Check Lattice.__init__ processing of arguments.
+        '''
+        self.assertRaises(ValueError, Lattice,
+                          self.lattice, c=4)
+        self.assertRaises(ValueError, Lattice,
+                          base=self.lattice.base,
+                          baserot=self.lattice.baserot)
+        self.assertRaises(ValueError, Lattice, 1, 2, 3)
+        self.assertRaises(ValueError, Lattice, 1, 2, 3, 80, 90)
+        L0 = self.lattice
+        L0.setLatBase(L0.cartesian([[1, 1, 0], [0, 1, 1], [1, 0, 1]]))
+        L1 = Lattice(L0)
+        self.assertTrue(numpy.array_equal(L0.base, L1.base))
+        L2 = Lattice(base=L0.base)
+        self.assertTrue(numpy.array_equal(L0.base, L2.base))
+        L3 = Lattice(*L0.abcABG(), baserot=L0.baserot)
+        self.assertTrue(numpy.allclose(L0.base, L3.base))
+        return
+
+
     def test_setLatPar(self):
         """check calculation of standard unit cell vectors"""
         from numpy import dot
@@ -146,8 +169,6 @@ class TestLattice(unittest.TestCase):
 
     def test_setLatBase(self):
         """check calculation of unit cell rotation"""
-        import numpy
-        import numpy.linalg as numalg
         base = numpy.array([[ 1.0,  1.0,  0.0],
                           [ 0.0,  1.0,  1.0],
                           [ 1.0,  0.0,  1.0]])
@@ -175,6 +196,93 @@ class TestLattice(unittest.TestCase):
                 [[1, 0, 0], [0,0,1], [0,1,0]])
         return
 
+
+    def test_reciprocal(self):
+        '''check calculation of reciprocal lattice.'''
+        r1 = self.lattice.reciprocal()
+        self.assertEqual((1, 1, 1, 90, 90, 90), r1.abcABG())
+        L2 = Lattice(2, 4, 8, 90, 90, 90)
+        r2 = L2.reciprocal()
+        self.assertEqual((0.5, 0.25, 0.125, 90, 90, 90), r2.abcABG())
+        rr2 = r2.reciprocal()
+        self.assertTrue(numpy.array_equal(L2.base, rr2.base))
+        return
+
+
+    def test_dot(self):
+        '''check dot product of lattice vectors.'''
+        L = self.lattice
+        L.setLatPar(gamma=120)
+        self.assertAlmostEqual(-0.5, L.dot([1, 0, 0], [0, 1, 0]), self.places)
+        va5 = numpy.tile([1.0, 0.0, 0.0], (5, 1))
+        vb5 = numpy.tile([0.0, 1.0, 0.0], (5, 1))
+        self.assertTrue(numpy.array_equal(5 * [-0.5], L.dot(va5, vb5)))
+        self.assertTrue(numpy.array_equal(5 * [-0.5], L.dot(va5[0], vb5)))
+        self.assertTrue(numpy.array_equal(5 * [-0.5], L.dot(va5, vb5[0])))
+        return
+
+
+    def test_norm(self):
+        '''check norm of a lattice vector.'''
+        self.assertEqual(1, self.lattice.norm([1, 0, 0]))
+        u = numpy.array([[3, 4, 0], [1, 1, 1]])
+        self.assertListAlmostEqual([5, 3**0.5], self.lattice.norm(u))
+        self.lattice.setLatPar(gamma=120)
+        self.assertAlmostEqual(1, self.lattice.norm([1, 1, 0]), self.places)
+        return
+
+
+    def test_rnorm(self):
+        '''check norm of a reciprocal vector.'''
+        L = self.lattice
+        L.setLatPar(1, 1.5, 2.3, 80, 95, 115)
+        r = L.reciprocal()
+        hkl = [0.5, 0.3, 0.2]
+        self.assertAlmostEqual(r.norm(hkl), L.rnorm(hkl), self.places)
+        hkl5 = numpy.tile(hkl, (5, 1))
+        self.assertListAlmostEqual(5 * [r.norm(hkl)], L.rnorm(hkl5))
+        return
+
+
+    def test_dist(self):
+        '''check dist function for distance between lattice points.'''
+        L = self.lattice
+        L.setLatPar(1, 1.5, 2.3, 80, 95, 115)
+        u = [0.1, 0.3, 0.7]
+        v = [0.3, 0.7, 0.7]
+        d0 = numalg.norm(L.cartesian(numpy.array(u) - v))
+        self.assertAlmostEqual(d0, L.dist(u, v), self.places)
+        self.assertAlmostEqual(d0, L.dist(v, u), self.places)
+        u5 = numpy.tile(u, (5, 1))
+        v5 = numpy.tile(v, (5, 1))
+        self.assertListAlmostEqual(5 * [d0], L.dist(u, v5))
+        self.assertListAlmostEqual(5 * [d0], L.dist(u5, v))
+        self.assertListAlmostEqual(5 * [d0], L.dist(v5, u5))
+        return
+
+
+    def test_angle(self):
+        '''check angle calculation between lattice vectors.'''
+        from math import degrees, acos
+        L = self.lattice
+        L.setLatPar(1, 1.5, 2.3, 80, 95, 115)
+        u = [0.1, 0.3, 0.7]
+        v = [0.3, 0.7, 0.7]
+        uc = L.cartesian(u)
+        vc = L.cartesian(v)
+        a0 = degrees(acos(numpy.dot(uc, vc) /
+                          (numalg.norm(uc) * numalg.norm(vc))))
+        self.assertAlmostEqual(a0, L.angle(u, v), self.places)
+        self.assertAlmostEqual(a0, L.angle(v, u), self.places)
+        u5 = numpy.tile(u, (5, 1))
+        v5 = numpy.tile(v, (5, 1))
+        self.assertListAlmostEqual(5 * [a0], L.angle(u, v5))
+        self.assertListAlmostEqual(5 * [a0], L.angle(u5, v))
+        self.assertListAlmostEqual(5 * [a0], L.angle(v5, u5))
+        return
+
+
+
     def test_repr(self):
         """check string representation of this lattice"""
         r = repr(self.lattice)
